﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Syncfusion.Windows.Chart;
using Syncfusion.Windows.Shared;
using Syncfusion.Windows.Collections;
using ASF.ES.Mistral.VisuGDT.InfoLayer;
using System.Collections.ObjectModel;

namespace ASF.ES.Mistral.VisuGDT.UILayer
{
    /// <summary>
    /// Logique d'interaction pour CourbesUC.xaml
    /// </summary>
    public partial class CourbesUC : UserControl, IInitialisable    
    {

        #region Paramètre de l'UC
        private String _libelleCourbe;
        /// <summary>
        /// Gets or sets the libelleCourbe property
        /// </summary>
        public String LibelleCourbe
        {
            [global::System.Diagnostics.DebuggerStepThrough]
            get
            {
                return _libelleCourbe;
            }
            [global::System.Diagnostics.DebuggerStepThrough]
            set
            {
                _libelleCourbe = value;
            }
        }
        private Int32 _numeroCourbe;
        /// <summary>
        /// Gets or sets the numeroCourbe property
        /// </summary>  
        public Int32 NumeroCourbe
        {
            [global::System.Diagnostics.DebuggerStepThrough]
            get
            {
                return _numeroCourbe;
            }
            [global::System.Diagnostics.DebuggerStepThrough]
            set
            {
                _numeroCourbe = value;
            }
        }
        private OuvertureParametrageAffichageCourbeType _typeOuvertureCourbe;
        /// <summary>
        /// Gets or sets the typeOuvertureCourbe property
        /// </summary>
        public OuvertureParametrageAffichageCourbeType TypeOuvertureCourbe
        {
            [global::System.Diagnostics.DebuggerStepThrough]
            get
            {
                return _typeOuvertureCourbe;
            }
            [global::System.Diagnostics.DebuggerStepThrough]
            set
            {
                _typeOuvertureCourbe = value;
            }
        }
        private TypeCourbeEnum _typeCourbe;
        /// <summary>
        /// Gets or sets the typeCourbe property
        /// </summary>  
        public TypeCourbeEnum TypeCourbe
        {
            [global::System.Diagnostics.DebuggerStepThrough]
            get
            {
                return _typeCourbe;
            }
            [global::System.Diagnostics.DebuggerStepThrough]
            set
            {
                _typeCourbe = value;
            }
        }
        #endregion

        public CourbesUC()
        {
            InitializeComponent();
        }


        // Définition des axes
        private ChartAxis _primaryAxis;
        private ChartAxis _secondaryAxis;
        private ChartAxis _thirdAxis;

        #region IInitialisable Membres
        public void Initialiser()
        {
            if (TypeOuvertureCourbe == OuvertureParametrageAffichageCourbeType.TempsReel)
            {
                TitreFenTBl.Text = "Courbe Temps Réel - " + TypeConverterHelper.ConvertTypeCourbeToString(TypeCourbe);
            }
            else
            {
                TitreFenTBl.Text = "Courbe Historique - " + TypeConverterHelper.ConvertTypeCourbeToString(TypeCourbe);
            }

            TitreGraphTBl.Text = LibelleCourbe;

            // Couleur des courbes
            Brush seriePrevuBsh = Brushes.Green;
            Brush serieRealiseBsh = Brushes.Blue;
            Brush serieQTV6Bsh = (Brush)App.Current.TryFindResource("BrushDebit6Mn");
            Brush serieQVL6Bsh = Brushes.Maroon;
            Brush serieQVC6Bsh = Brushes.MediumBlue;
            Brush serieQTVHBsh = (Brush)App.Current.TryFindResource("BrushDebitH");
            Brush serieQVLHBsh = Brushes.Maroon;
            Brush serieQVCHBsh = Brushes.MediumBlue;
            Brush serieVitesseBsh = ((Brush)App.Current.TryFindResource("BrushVitesse"));
            Brush serieTauxBsh = ((Brush)App.Current.TryFindResource("BrushTaux"));

            try
            {
                CourbeTempsReelWPFFacade courbeFacade = CourbeTempsReelWPFFacadeHelper.Get(NumeroCourbe);
                if (courbeFacade != null)
                {
                    // Mise en place du Binding
                    CourbeSFCh.DataContext = courbeFacade;

                    // Propriétés générales du chart qui nous intéressent
                    CourbeSFCh.Foreground = Brushes.Black;
                    CourbeSFCh.Background = Brushes.White;
                    CourbeSFCh.BorderBrush = Brushes.Black;

                    // Création de l'area qui contiendra notre graphique
                    _primaryAxis = CourbeSFCh.Areas[0].PrimaryAxis;
                    //_primaryAxis.ValueType = ChartValueType.Double;
                    _primaryAxis.Range = new DoubleRange(0, courbeFacade.Profondeur * 60);
                    _primaryAxis.LabelsMode = ChartAxisLabelsMode.Custom;

                    for (int iProfLoop = 2; iProfLoop <= courbeFacade.Profondeur; iProfLoop+=2)
                    {
                        _primaryAxis.CustomLabels.Add(new ChartAxisLabel());
                        _primaryAxis.CustomLabels[_primaryAxis.CustomLabels.Count - 1].Content = iProfLoop.ToString();
                        _primaryAxis.CustomLabels[_primaryAxis.CustomLabels.Count - 1].Position = iProfLoop * 60;
                    }

                    _primaryAxis.Interval = 120;
                    _primaryAxis.IsAutoSetRange = false;
                    //_primaryAxis.Header = "h";
                    //_primaryAxis.HeaderAlignment = ChartAlignment.Far;
                    _secondaryAxis = CourbeSFCh.Areas[0].SecondaryAxis;
                    //_secondaryAxis.Range = new DoubleRange(0, 1500);
                    //_secondaryAxis.Interval = 250;
                    _secondaryAxis.IsAutoSetRange = true;
                    //_secondaryAxis.Header = "v/h";
                    //_secondaryAxis.HeaderAlignment = ChartAlignment.Near;
                    CourbeSFCh.Areas[0].Background = Brushes.White;
                    CourbeSFCh.Areas[0].BorderBrush = Brushes.White;

                    //Mise en place de la légende
                    ChartLegend chLegend = new ChartLegend();
                    chLegend.BorderBrush = Brushes.Transparent;
                    chLegend.Background = Brushes.White;
                    chLegend.CornerRadius = new CornerRadius(4);
                    chLegend.BorderThickness = new Thickness(1);
                    chLegend.Foreground = Brushes.Black;
                    chLegend.IconHeight = 10;
                    chLegend.IconWidth = 10;
                    chLegend.ColumnsCount = 2;
                    chLegend.RowsCount = 1;
                    chLegend.HorizontalAlignment = HorizontalAlignment.Left;
                    CourbeSFCh.Legends.Add(chLegend);

                    switch (TypeCourbe)
                    {
                        // Cas où il y a 2 axes
                        case TypeCourbeEnum.TraficPrevuRealise:
                            TitreGraphTBl.DataContext = courbeFacade.PointMesureWPFFacade[0];
                            InitChartSeries(
                                courbeFacade.DetailsCourbeWPFFacade[courbeFacade.PointMesureWPFFacade[0].RefPM],
                                "NumOrdre",
                                "QTVHoraire",
                                ChartTypes.Spline,
                                "Réalisé",
                                _primaryAxis,
                                _secondaryAxis,
                                seriePrevuBsh);
                            break;
                        case TypeCourbeEnum.EvolutionDebits6Mn:
                            TitreGraphTBl.DataContext = courbeFacade.PointMesureWPFFacade[0];
                            break;
                        case TypeCourbeEnum.EvolutionDebitsHoraire:
                            TitreGraphTBl.DataContext = courbeFacade.PointMesureWPFFacade[0];
                            break;
                        case TypeCourbeEnum.EvolutionQ6MnTousVehicules:
                            TitreGraphTBl.DataContext = TypeConverterHelper.ConvertTypeCourbeToString(TypeCourbeEnum.EvolutionQ6MnTousVehicules);
                            break;
                        case TypeCourbeEnum.EvolutionVitesse6Mn:
                            TitreGraphTBl.DataContext = TypeConverterHelper.ConvertTypeCourbeToString(TypeCourbeEnum.EvolutionQ6MnTousVehicules);
                            break;
                        // Cas où il y a 3 axes
                        case TypeCourbeEnum.EvolutionQToV:
                            TitreGraphTBl.DataContext = courbeFacade.PointMesureWPFFacade[0];
                            // pour mettre un troisième axe sur le chart
                            _thirdAxis = new ChartAxis();
                            _thirdAxis.IsAutoSetRange = true;
                            _thirdAxis.Orientation = Orientation.Vertical;
                            _thirdAxis.OpposedPosition = true;
                            _thirdAxis.Interval = 25;
                            //_thirdaxis.Header = "km/h - %";
                            //_thirdaxis.HeaderAlignment = ChartAlignment.Near;
                            CourbeSFCh.Areas[0].Axes.Add(_thirdAxis);

                            InitChartSeries(
                                courbeFacade.DetailsCourbeWPFFacade[courbeFacade.PointMesureWPFFacade[0].RefPM],
                                "NumOrdre",
                                "QTV6Mn",
                                ChartTypes.Spline,
                                "QTV6Mn",
                                _primaryAxis,
                                _secondaryAxis,
                                serieQTV6Bsh);

                            InitChartSeries(
                                courbeFacade.DetailsCourbeWPFFacade[courbeFacade.PointMesureWPFFacade[0].RefPM],
                                "NumOrdre",
                                "QTVHoraire",
                                ChartTypes.Spline,
                                "QTVHoraire",
                                _primaryAxis,
                                _secondaryAxis,
                                serieQTVHBsh);

                            InitChartSeries(
                                courbeFacade.DetailsCourbeWPFFacade[courbeFacade.PointMesureWPFFacade[0].RefPM],
                                "NumOrdre",
                                "Vitesse6Mn",
                                ChartTypes.Spline,
                                "Vitesse",
                                _primaryAxis,
                                _thirdAxis,
                                serieVitesseBsh);

                            InitChartSeries(
                                courbeFacade.DetailsCourbeWPFFacade[courbeFacade.PointMesureWPFFacade[0].RefPM],
                                "NumOrdre",
                                "Taux6Mn",
                                ChartTypes.Spline,
                                "Taux",
                                _primaryAxis,
                                _thirdAxis,
                                serieTauxBsh);


                            break;
                    }

                    //RemplirDonnees();

                    // on peut déclarer les évèmenents sur les courbes pour le survol avec la souris
                    foreach (ChartSeries chSeriesLoop in CourbeSFCh.Areas[0].Series)
                    {
                        chSeriesLoop.MouseMove += new ChartMouseEventHandler(OnCourbesUC_MouseMove);
                    }
                }
            } catch (Exception Exc)
            {
                AffichageMessageHelper.Erreur(Exc.Message);
            }
        }

        private void InitChartSeries(
                    ObservableCollection<DetailCourbeWPFFacade> detailFacade,
                    String pathX,
                    String pathY,
                    ChartTypes typeChart,
                    string titreLegende,
                    ChartAxis axeX,
                    ChartAxis axeY,
                    Brush couleurSerie)
        {
            try
            {
                ChartSeries chartSerie = new ChartSeries(typeChart);
                CourbeSFCh.Areas[0].Series.Add(chartSerie);
                chartSerie.DataSource = detailFacade;
                chartSerie.IsIndexed = false;
                chartSerie.BindingPathX = pathX;
                chartSerie.BindingPathsY = new string[] { pathY };
                chartSerie.Label = titreLegende;
                chartSerie.StrokeThickness = 3;
                chartSerie.XAxis = axeX;
                chartSerie.YAxis = axeY;
                chartSerie.Interior = couleurSerie;
            } catch (Exception Exc)
            {
                AffichageMessageHelper.Erreur(String.Format("Problème à la création de la courbe {0}. ", titreLegende));
            }
        }
        #endregion

        //[Obsolete ("Pour maquette, à supprimer")]
        //private  void RemplirDonnees()
        //{
        //    IList<ChartValuesDTO> data = new List<ChartValuesDTO>();
        //    IList<ChartValuesDTO> data2 = new List<ChartValuesDTO>();

        //    if (TypeCourbe == TypeCourbeEnum.EvolutionQToV)
        //    {
        //        IList<ChartValuesDTO> data3 = new List<ChartValuesDTO>();
        //        IList<ChartValuesDTO> data4 = new List<ChartValuesDTO>();

        //        // courbe de démo Q6mn
        //        data.Add(new ChartValuesDTO(0, 3350));
        //        data.Add(new ChartValuesDTO(-1, 3550));
        //        data.Add(new ChartValuesDTO(-2, 3900));
        //        data.Add(new ChartValuesDTO(-3, 4180));
        //        data.Add(new ChartValuesDTO(-4, 4300));
        //        data.Add(new ChartValuesDTO(-5, 4400));
        //        data.Add(new ChartValuesDTO(-6, 3950));
        //        data.Add(new ChartValuesDTO(-7, 2500));
        //        data.Add(new ChartValuesDTO(-8, 1750));
        //        AjouterSeries(ChartTypes.Spline, data, "Q6mn (v/h)", _primaryAxis, _secondaryAxis);
        //        // courbe de démo QH
        //        data2.Add(new ChartValuesDTO(0, 3150));
        //        data2.Add(new ChartValuesDTO(-1, 3650));
        //        data2.Add(new ChartValuesDTO(-2, 3800));
        //        data2.Add(new ChartValuesDTO(-3, 4280));
        //        data2.Add(new ChartValuesDTO(-4, 4200));
        //        data2.Add(new ChartValuesDTO(-5, 4600));
        //        data2.Add(new ChartValuesDTO(-6, 3750));
        //        data2.Add(new ChartValuesDTO(-7, 2800));
        //        data2.Add(new ChartValuesDTO(-8, 2050));
        //        AjouterSeries(ChartTypes.Spline, data2, "QH (v/h)", _primaryAxis, _secondaryAxis);
        //        // courbe de démo V
        //        data3.Add(new ChartValuesDTO(0, 109));
        //        data3.Add(new ChartValuesDTO(-1, 108));
        //        data3.Add(new ChartValuesDTO(-2, 105));
        //        data3.Add(new ChartValuesDTO(-3, 104));
        //        data3.Add(new ChartValuesDTO(-4, 108));
        //        data3.Add(new ChartValuesDTO(-5, 110));
        //        data3.Add(new ChartValuesDTO(-6, 118));
        //        data3.Add(new ChartValuesDTO(-7, 114));
        //        data3.Add(new ChartValuesDTO(-8, 125));
        //        AjouterSeries(ChartTypes.Spline, data3, "V (km/h)", _primaryAxis, _thirdaxis);
        //        // courbe de démo To
        //        data4.Add(new ChartValuesDTO(0, 11));
        //        data4.Add(new ChartValuesDTO(-1, 11));
        //        data4.Add(new ChartValuesDTO(-2, 14));
        //        data4.Add(new ChartValuesDTO(-3, 14));
        //        data4.Add(new ChartValuesDTO(-4, 14));
        //        data4.Add(new ChartValuesDTO(-5, 14));
        //        data4.Add(new ChartValuesDTO(-6, 14));
        //        data4.Add(new ChartValuesDTO(-7, 11));
        //        data4.Add(new ChartValuesDTO(-8, 11));
        //        AjouterSeries(ChartTypes.Line, data4, "To (%)", _primaryAxis, _thirdaxis);
        //    }
        //    else
        //    {
        //        data.Add(new ChartValuesDTO(0, 900));
        //        data.Add(new ChartValuesDTO(1, 900));
        //        data.Add(new ChartValuesDTO(2, 900));
        //        data.Add(new ChartValuesDTO(3, 1000));
        //        data.Add(new ChartValuesDTO(4, 1100));
        //        data.Add(new ChartValuesDTO(5, 1500));
        //        data.Add(new ChartValuesDTO(6, 2050));
        //        data.Add(new ChartValuesDTO(7, 2800));
        //        data.Add(new ChartValuesDTO(8, 3550));
        //        data.Add(new ChartValuesDTO(9, 4500));
        //        data.Add(new ChartValuesDTO(10, 5000));
        //        data.Add(new ChartValuesDTO(11, 5200));
        //        data.Add(new ChartValuesDTO(12, 4850));
        //        data.Add(new ChartValuesDTO(13, 4500));
        //        data.Add(new ChartValuesDTO(14, 4600));
        //        data.Add(new ChartValuesDTO(15, 4950));
        //        data.Add(new ChartValuesDTO(16, 4850));
        //        data.Add(new ChartValuesDTO(17, 4600));
        //        data.Add(new ChartValuesDTO(18, 4200));
        //        data.Add(new ChartValuesDTO(19, 3600));
        //        data.Add(new ChartValuesDTO(20, 2950));
        //        data.Add(new ChartValuesDTO(21, 2100));
        //        data.Add(new ChartValuesDTO(22, 1650));
        //        data.Add(new ChartValuesDTO(23, 1300));
        //        AjouterSeries(ChartTypes.Spline, data, "Prévu", _primaryAxis, _secondaryAxis);
        //        data2.Add(new ChartValuesDTO(0, 880));
        //        data2.Add(new ChartValuesDTO(1, 880));
        //        data2.Add(new ChartValuesDTO(2, 880));
        //        data2.Add(new ChartValuesDTO(3, 950));
        //        data2.Add(new ChartValuesDTO(4, 1100));
        //        data2.Add(new ChartValuesDTO(5, 1480));
        //        data2.Add(new ChartValuesDTO(6, 1850));
        //        data2.Add(new ChartValuesDTO(7, 2500));
        //        data2.Add(new ChartValuesDTO(8, 3200));
        //        data2.Add(new ChartValuesDTO(9, 4000));
        //        data2.Add(new ChartValuesDTO(10, 4550));
        //        data2.Add(new ChartValuesDTO(11, 4700));
        //        AjouterSeries(ChartTypes.Spline, data2, "Réalisé", _primaryAxis, _secondaryAxis);
        //    }
        //}

        void OnCourbesUC_MouseMove(object sender, ChartMouseEventArgs e)
        {
            ChartSeries chSerie = sender as ChartSeries;
            CourbeSFCh.ToolTip = "{" + e.Segment.XDataMeasure.Median.ToString("0.00") + "} - {" + e.Segment.YDataMeasure.Median.ToString("0.00") + "}";
        }
        //private void AjouterSeries(
        //    ChartTypes typeChart, 
        //    IList<ChartValuesDTO> data, 
        //    string titreLegende, 
        //    ChartAxis axeX, 
        //    ChartAxis axeY)
        //{
        //    try {
        //        ChartSeries chartSerie = new ChartSeries(typeChart);
        //        CourbeSFCh.Areas[0].Series.Add(chartSerie);
        //        chartSerie.DataSource = data;
        //        chartSerie.IsIndexed = false;
        //        chartSerie.BindingPathX = "X";
        //        chartSerie.BindingPathsY = new string[] { "Y" };
        //        chartSerie.Label = titreLegende;
        //        chartSerie.StrokeThickness = 3;
        //        chartSerie.XAxis = axeX;
        //        chartSerie.YAxis = axeY;
        //    } catch (Exception Exc) {
        //        AffichageMessageHelper.Erreur(String.Format("Problème à la création de la courbe {0}. ", titreLegende));
        //        Logger.Error(String.Format("(CourbesUC.AjouterSeries()) - Problème à la création de la courbe {0}. ",titreLegende), Exc);
        //    }
        //}

        
    }
}
